// Author: Gerry O'Rourke
// Date 12/05/2020  Version 1.0
// Date 12/03/2021  Version 1.1
//
// Changed Butons to be Active once recorder is connected - rather than based on Agent State.
// This is due to UCCX only has TALKING state when CCX call is routed, and not on all calls.
// This could be fixed easily on CCE but more work on CCX.
//
// Version 1.1
// Fix1: extension + Space after a keepalive failure was adding additional space - and not finding extension. This has been fixed by adding the space as soon as we get the extension.
// Fix2: If the  keepalive fails, then show the login button but also retry automatically to log back in up for up to the max number of keepalives (3 by default)
// Fix3: If Supression Fails - show login and diable keepalives - agent can login


    /*
    MIT License
 
    Copyright (c) 2021 Gerard O'Rourke
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
 
    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.
 
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
    */


var finesse = finesse || {};
finesse.gadget = finesse.gadget || {};
finesse.container = finesse.container || {};
_clientLogger = finesse.cslogger.ClientLogger || {};  // for logging


/** @namespace */
finesse.modules = finesse.modules || {};
finesse.modules.RedboxCallSuppressionGadget = (function ($) {

    var _user;
    var _apiusername = 'redboxapi';
    var _apipassword = 'DVgTLKRc9VAt4h2T';
    var _host1 = 'redbox1.mydomain.com';
    var _baseURL = 'http://' + _host1 + ':1480';
    var _loginPath = '/api/v1/sessions/login';
    var _logoutPath = '/api/v1/sessions/logout';
    var _deviceListPath = '/api/v1/config/devices';
    var _suppressStartPath = '/api/v1/suppression/start/device';
    var _suppressStopPath = '/api/v1/suppression/stop/device';
    var _keepalivePath = '/api/v1/sessions/keepAlive';
    var _authToken;
    var _myExtension;
    var _myDeviceId = []; //Array for the Device IDs - as the recorder could have more than one.
    var _myDeviceIdCounter;
    var _connected = false;
    var _connectionStatus = 'not connected';
    var _keepaliveMsg = "";
    var _keepaliveCounter = 0;
    var _keepaliveMaxCounter = 3;
    var _keepaliveInterval = 150000; // defaults to 150000, i.e. 2.5 mins. The Redbox Session timer is 5 mins - so keepalives need to be set before this 5 min timer expires.
    var _userState = "";
    var _version = '1.1';

    /**
     * Populates the fields in the gadget with data
     */
    _updateUI = function () {
        _clientLogger.log ("_updateUI(): --------");

        if(_connected) {
            _clientLogger.log ("We are connected - Enable Pause / Resume buttons");
            $('#startSuppressionButton').removeClass("disabled");
            $('#startSuppressionButton').prop('disabled', false);

            $('#stopSuppressionButton').removeClass("disabled");
            $('#stopSuppressionButton').prop('disabled', false);
        }   else {
                _clientLogger.log ("_updateUI(): We are no longer connected to the recorder or do NOT have the Redbox Device ID - do NOT enable the START or STOP suppression buttons");

                $('#startSuppressionButton').addClass("disabled");
                $('#startSuppressionButton').prop('disabled', true);

                $('#stopSuppressionButton').addClass("disabled");
                $('#stopSuppressionButton').prop('disabled', true);
				
				$("#status").text(_connectionStatus);
				$('#loginButton').show(); //Show Login Button
            }
        gadgets.window.adjustHeight();
    },
    
        
    /* This function uses the gadgets.io.makeRequest - to do the REST GET, POST or PUT requests and returns success / error
     * Reference: https://developers.google.com/gadgets/docs/reference#makerequest
     * 
     * @param {type} url
     * @param {type} options
     * @param {type} handlers
     * @returns {undefined}
     */
    makeRequest = function (url, options, handlers) {
        var params, uuid;
        _clientLogger.log("makeRequest()");

        // Protect against null dereferencing of options & handlers allowing its (nonexistant) keys to be read as undefined
        params = {};
        options = options || {};
        handlers.success = _util.validateHandler(handlers.success);
        handlers.error = _util.validateHandler(handlers.error);

        // Request Headers
        params[gadgets.io.RequestParameters.HEADERS] = {};

        // HTTP method is a passthrough to gadgets.io.makeRequest
        params[gadgets.io.RequestParameters.METHOD] = options.method;

        if (options.method === "GET") {
            // Disable caching for GETs
            if (url.indexOf("?") > -1) {
                url += "&";
            } else {
                url += "?";
            }
            url += "nocache=" + _util.currentTimeMillis();
            uuid = _util.generateUUID();
            params[gadgets.io.RequestParameters.HEADERS].requestId = uuid;
            params[gadgets.io.RequestParameters.GET_FULL_HEADERS] = "true";
			
        } else {
            // Generate a requestID and add it to the headers
            uuid = _util.generateUUID();
            params[gadgets.io.RequestParameters.HEADERS].requestId = uuid;
            params[gadgets.io.RequestParameters.GET_FULL_HEADERS] = "true";
        }
        
        // Add authorization to the request header if provided (not used in this app - was in the Sample REST API)
        if(options.authorization) {
            params[gadgets.io.RequestParameters.HEADERS].Authorization = options.authorization;
        }

        if(options.login) {
            params[gadgets.io.RequestParameters.HEADERS]["username"] = _apiusername;
            params[gadgets.io.RequestParameters.HEADERS]["password"] = _apipassword;
        }

        if(options.authToken) {
            params[gadgets.io.RequestParameters.HEADERS]["authToken"] = _authToken;
        }

        // Add content type & body if content body is provided
        if (options.content) {
            // Content Type
            params[gadgets.io.RequestParameters.HEADERS]["Content-Type"] = options.contentType;
            // Content
            params[gadgets.io.RequestParameters.POST_DATA] = options.content;
        }

        // Call the gadgets.io.makereqest function with the encoded url
        _clientLogger.log("makeRequest(): Making a REST API request to: " + url);
        gadgets.io.makeRequest(encodeURI(url), handleResponse(handlers), params);
    },
    
    /**
     * Handler for the response of the REST API request. This function determines if
     * the success or error handler should be called based on HTTP status code.
     *
     * @param {Object} handlers
     *     An object containing the success and error handlers.
     *
     *     {Function} handlers.success(response)
     *        A callback function to be invoked for a successful request.
     *     {Function} handlers.error(response)
     *        A callback function to be invoked for an unsuccessful request.
     */
    handleResponse = function(handlers) {
        return function (response) {
            _clientLogger.log("handleResponse(): The response status code is: " + response.rc);

            // Send the response to the success handler if the http status
            // code is 200 - 299. Send the response to the error handler
            // otherwise.
            if (response.rc >= 200 && response.rc < 300 && handlers.success) {
                _clientLogger.log("handleResponse(): Got a successful response.");
                handlers.success(response);
            } else if (handlers.error) {
                _clientLogger.log("handleResponse(): Got a failure response.");
                handlers.error(response);
            } else {
                _clientLogger.log("handleResponse(): Missing the success and/or error handler.");
            }
        };
    },
    

    /**
     * Handler for login when successful.
     */
    _loginSuccess = function(response) {
        _clientLogger.log ("_loginSuccess(): --------");
        var _loginSuccessJSON = JSON.parse(response.text);
        //_clientLogger.log ("_loginSuccess(): _loginSuccessJSON: " + JSON.stringify(_loginSuccessJSON)); // Commenting this out to remove authToken from Console Logs

        _authToken = _loginSuccessJSON.authToken;
        //_clientLogger.log ("_loginSuccess(): authToken: " + _authToken); // Commenting this out to remove authToken from Console Logs

        $('#loginButton').hide(); //Hide Login Button

        _connectionStatus = 'Logged In'
        _keepaliveCounter = 0;
        $("#status").text(_connectionStatus);


        _deviceList();
    },

    /**
     * Handler for login Errors
     */
    _loginError = function(response) {
        _clientLogger.log ("_loginError(): in method");
        var statusCode = response.rc;
        var errorMessage = 'Network or Configuration Error'; //Default Error

        $('#loginButton').show(); //Show Login Button
        _connectionStatus = 'Not Logged In'
        $("#status").text(_connectionStatus);

        // Checking if we got any HTTP Headers in response (maybe not if there was a Timeout etc.)
        if (typeof response.headers != "undefined") {
            var myContentType = response.headers["content-type"];
            console.log("Content-Type: " + myContentType);

            //Checking if Content is JSON and so use the error value returned by the server
            if (myContentType == "application/json; charset=UTF-8") {
                var responseJSON = JSON.parse(response.text);
                var errorMessage = responseJSON.error;
            }else{
                _clientLogger.log ("Content-Type is NOT \'application/json; charset=UTF-8\'");
                errorMessage = 'Server Error';
            }
        }

        _clientLogger.log ("_loginError() - WebServerStatusCode: " + statusCode);
        _clientLogger.log ("_loginError() - errorMessage: " + errorMessage);


        $('#errorMsgHeader').text("Failed to Login to Recorder!");
        $('#errorMsgCode').text(statusCode);
        $('#errorMsgMessage').text(errorMessage);
        $('#errorMsg').show();

        gadgets.window.adjustHeight();
    },


    /**
     * Handler for logout when successful.
     * NOTE - IT LOOKS LIKE FINESSE WON'T WAIT FOR THIS AS ITS LOGGING OUT - SO THIS EVENT HANDLER WILL NEVER ACTUALLY BE TRIGGERED
     * BUT THE MAIN THING IS WE SENT A LOGOUT REQUEST!
     */ 
    _logoutSuccess = function(response) {
        _clientLogger.log ("_logoutSuccess(): --------");
    },
    

    /**
     * Handler for logout when successful.
     * NOTE - IT LOOKS LIKE FINESSE WON'T WAIT FOR THIS AS ITS LOGGING OUT - SO THIS EVENT HANDLER WILL NEVER ACTUALLY BE TRIGGERED
     * BUT THE MAIN THING IS WE SENT A LOGOUT REQUEST!
     */
    _logoutError = function(response) {
        _clientLogger.log ("_logoutError(): --------");
        _clientLogger.log ("_logoutError(): Did not log out cleanly!");
    },


    /**
     * Handler for DEVICE LIST when successful.
     */
    _deviceListSuccess = function(response) {
        _clientLogger.log ("_deviceListSuccess(): --------");
        _connectionStatus = 'Getting Device List...';
        $("#status").text(_connectionStatus);


        var _deviceListJSON = JSON.parse(response.text);
        _clientLogger.log ("_deviceListSuccess(): DeviceList: " + JSON.stringify(_deviceListJSON));

        _clientLogger.log ("_deviceListSuccess(): Will Search the Device List for my Extension: " + _myExtension);
        _clientLogger.log ("_deviceListSuccess(): DeviceLength: " + _deviceListJSON.devices.length);

        _myDeviceIdCounter = 0;
		_myDeviceId = []; //clear Array - when DeviceList has been run after an error

        for (var i = 0; i < _deviceListJSON.devices.length; i++) {
            var device = _deviceListJSON.devices[i];

            //	if (device.extension === "12022 (SEP38ED1855072B)"){
            // Note - the String.ptototype.includes is not available in IE11 so using a Polyfill
            // Reference: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/includes
            // Download link: https://www.npmjs.com/package/string.prototype.includes

            if (device.extension.includes(_myExtension)){
                //_myDeviceId = device.deviceID; //Original Non Array version - single extension
                _myDeviceId.push(device.deviceID); // This pushs the value into the Array "_myDeviceId"
                _clientLogger.log("Found Device ID: " + (_myDeviceId[_myDeviceIdCounter]));

                _myDeviceIdCounter++;
                $("#deviceId").text(_myDeviceId.toString());
                _connected = true; // Since we found a Device ID - we set _connected to true!
                //break; //unable to stop on First match as Redbox can have mutiple devices with same extenion - we have to find them all - a PAIN!
            }
        }

        _clientLogger.log("My Device ID Array is: " + _myDeviceId.toString());

        if (_connected) {
            _connectionStatus = "Connected";
            _updateUI();


            // Show Keep-Alive Here - we won't actually do a keepalive until the keepalive timer expires.
            _keepaliveMsg = moment().format('LTS');
            
            $("#keepalive").text(_keepaliveMsg);

            _clientLogger.log("Keepalive Timer started! Keepalive in " + (_keepaliveInterval/1000) + " seconds...");
            setTimeout(function () {
                _keepalive();
            }, _keepaliveInterval);
        }else{
            _connectionStatus = "Unable to get Device ID";
            $('#errorMsgHeader').text("Device List Error");
            $('#errorMsgCode').text("Unable to find this extension on the Recorder");
            $('#errorMsgMessage').text("Contact your administrator");
            $('#errorMsg').show();
            gadgets.window.adjustHeight();
        }
        $("#status").text(_connectionStatus);
    },


    /**
     * Handler for deviceList Server Errors
     */
    _deviceListError = function(response) {
        var statusCode = response.rc;
        var errorMessage = 'Network or Configuration Error'; //Default Error

        $('#loginButton').show(); //Hide Login Button
        _connectionStatus = 'Not Logged In - Unable to get Device List'
        $("#status").text(_connectionStatus);

        // Checking if we got any HTTP Headers in response (maybe not if there was a Timeout etc.)
        if (typeof response.headers != "undefined") {
            var myContentType = response.headers["content-type"];
            console.log("Content-Type: " + myContentType);

            //Checking if Content is JSON and so use the error value returned by the server
            if (myContentType == "application/json; charset=UTF-8") {
                var responseJSON = JSON.parse(response.text);
                var errorMessage = responseJSON.error;
            }else{
                _clientLogger.log ("Content-Type is NOT \'application/json; charset=UTF-8\'");
                errorMessage = 'Server Error';
            }
        }

        _clientLogger.log ("_loginError() - WebServerStatusCode: " + statusCode);
        _clientLogger.log ("_loginError() - errorMessage: " + errorMessage);

        $('#errorMsgHeader').text("Failed to Get Device List from Recorder!");
        $('#errorMsgCode').text(statusCode);
        $('#errorMsgMessage').text(errorMessage);
        $('#errorMsg').show();

        gadgets.window.adjustHeight();
    },

    /*
     * Handler for makeCall when successful.
     */
    _startSuppressionSuccess = function(response) {
        _clientLogger.log ("_startSuppressionSuccess(): --------");

        // If we have looped through and started Suppression on all our Device ID
        if ((_myDeviceIdCounter+1) >= _myDeviceId.length){
            _clientLogger.log ("_startSuppressionSuccess(): Successfully Suppressed all DeviceIDs");
            $('#startSuppressSuccessMsg').fadeIn("slow");
            gadgets.window.adjustHeight();            
            
            window.setTimeout(function () {
            $('#startSuppressSuccessMsg').fadeOut("slow",0, function() {gadgets.window.adjustHeight();});
        }, 3000);
    }else{
        _clientLogger.log ("_startSuppressionSuccess(): Starting Suppression on next Device ID");
        _myDeviceIdCounter++; // Add to our count and do a Start Suppression on our next DeviceID in our Array
        _startSuppression();
    }
    
    },
    
    /**
     * Handler for makeCall when error occurs.
     */
    _startSuppressionError = function(response) {
        _clientLogger.log ("_startSuppressionError(): --------");
        var statusCode = response.rc;
        var errorMessage = 'Network or Configuration Error. Contact your administrator or try again.'; //Default Error

        //$('#loginButton').show(); //Hide Login Button
        // Checking if we got any HTTP Headers in response (maybe not if there was a Timeout etc.)
        if (typeof response.headers != "undefined") {
            var myContentType = response.headers["content-type"];
            console.log("Content-Type: " + myContentType);

            //Checking if Content is JSON and so use the error value returned by the server
            if (myContentType == "application/json; charset=UTF-8") {
                var responseJSON = JSON.parse(response.text);
                var errorMessage = responseJSON.error;
            }else{
                _clientLogger.log ("Content-Type is NOT \'application/json; charset=UTF-8\'");
                errorMessage = 'Server Error. Contact your administrator or try again.';
            }
        }

        _clientLogger.log ("_startSuppressionError() - WebServerStatusCode: " + statusCode);
        _clientLogger.log ("_startSuppressionError() - errorMessage: " + errorMessage);

        $('#errorMsgHeader').text("Call Suppression Start Error");
        $('#errorMsgCode').text(statusCode);
        $('#errorMsgMessage').text(errorMessage);
        $('#errorMsg').show();
		_connected = false;
		_connectionStatus = "Disconnected - Suppression failed";
		_keepaliveCounter = _keepaliveMaxCounter; //Stop any further Keep-Alives

        if (statusCode==401){
            _clientLogger.log ("_startSuppressionError(): Failed due to 401 - Attempting to re-login");
            _login();
        }else{
            _updateUI();            
		}

        gadgets.window.adjustHeight();
	},


    _stopSuppressionSuccess = function(response) {
        _clientLogger.log ("_stopSuppressionSuccess(): --------");

       if ((_myDeviceIdCounter+1) >= _myDeviceId.length){
            _clientLogger.log ("_stopSuppressionSuccess(): Successfully Stop Suppression on all DeviceIDs");
            $('#stopSuppressSuccessMsg').fadeIn("slow");
            gadgets.window.adjustHeight();           
            window.setTimeout(function () {
            $('#stopSuppressSuccessMsg').fadeOut("slow",0, function() {gadgets.window.adjustHeight();});
        }, 3000);
        }else{
        _myDeviceIdCounter++;
        _stopSuppression();
        }      
    },

    /**
     * Handler for makeCall when error occurs.
     */
    _stopSuppressionError = function(response) {
        _clientLogger.log ("_stopSuppressionError(): --------");
        _connectionStatus = "Disconnected";
        $("#status").text(_connectionStatus);
		
        var statusCode = response.rc;
        var errorMessage = 'Network or Configuration Error. Contact your administrator or try again.'; //Default Error

        //$('#loginButton').show(); //Hide Login Button
        // Checking if we got any HTTP Headers in response (maybe not if there was a Timeout etc.)
        if (typeof response.headers != "undefined") {
            var myContentType = response.headers["content-type"];
            console.log("Content-Type: " + myContentType);

            //Checking if Content is JSON and so use the error value returned by the server
            if (myContentType == "application/json; charset=UTF-8") {
                var responseJSON = JSON.parse(response.text);
                var errorMessage = responseJSON.error;
            }else{
                _clientLogger.log ("Content-Type is NOT \'application/json; charset=UTF-8\'");
                errorMessage = 'Server Error. Contact your administrator or try again.';
            }
        }

        _clientLogger.log ("_stopSuppressionError() - WebServerStatusCode: " + statusCode);
        _clientLogger.log ("_stopSuppressionError() - errorMessage: " + errorMessage);

        $('#errorMsgHeader').text("Call Suppression Start Error");
        $('#errorMsgCode').text(statusCode);
        $('#errorMsgMessage').text(errorMessage);
        $('#errorMsg').show();
		_connected = false;
		_connectionStatus = "Disconnected - Suppression failed";
		_keepaliveCounter = _keepaliveMaxCounter; //Stop any further Keep-Alives

        if (statusCode==401){
            _clientLogger.log ("_stopSuppressionError(): Failed due to 401 - Attempting to re-login");
            _login();
        }else{
            _updateUI();            
		}
        gadgets.window.adjustHeight();
    },


    _keepaliveSuccess = function(response) {
        _clientLogger.log ("_keepaliveSuccess(): --------");
        _connectionStatus = "Connected";
        $("#status").text(_connectionStatus);
        

        _keepaliveMsg = moment().format('LTS');
        $("#keepalive").text(_keepaliveMsg);
        _keepaliveCounter = 0;
        setTimeout(function () {
            _keepalive();
        }, _keepaliveInterval);
    },

    /**
     * Handler for makeCall when error occurs.
     */
    _keepaliveError = function(response) {
        _clientLogger.log("_keepaliveError(): --------");
		_connected = false;
		_connectionStatus = "Disconnected - keepalive failed";

        $("#status").text(_connectionStatus);

        _keepaliveCounter++;
        _clientLogger.log("_keepaliveError(): " + _keepaliveCounter);

        var statusCode = response.rc;
        _clientLogger.log("_keepaliveError(): - WebServerStatusCode: " + statusCode);

        var errorMessage = 'unknown';

        // Checking if we got any HTTP Headers in response (maybe not if there was a Timeout etc.)
        if (typeof response.headers != "undefined") {
            var myContentType = response.headers["content-type"];
            console.log("Content-Type: " + myContentType);

            //Checking if Content is JSON and so use the error value returned by the server
            if (myContentType == "application/json; charset=UTF-8") {
                var responseJSON = JSON.parse(response.text);
                errorMessage = responseJSON.error;
                _clientLogger.log("_keepaliveError(): - errorMessage: " + errorMessage);
            } else {
                _clientLogger.log("_keepaliveError(): - errorMessage: " + errorMessage);
            }
        }

        _keepaliveMsg = moment().format('LTS');
        $("#keepalive").text("Failed: " + _keepaliveMsg + " - ErrorMessage: " + errorMessage);


        if (statusCode==401){
            _clientLogger.log ("_keepaliveError(): Failed due to 401 - Attempting to re-login");
            _connected = false;
            _connectionStatus = "Disconnected";
            _updateUI();            
            _login();
        }


        if (_keepaliveCounter < _keepaliveMaxCounter) {
            _clientLogger.log ("_keepaliveError(): Max counter not reached - retrying in " + (_keepaliveInterval/1000) + " seconds.");
            setTimeout(function () {
                _keepalive();
            }, _keepaliveInterval);
        }else{
            $("#keepalive").text("Keep-alive failed. No more attempts. Max counter reached");
        }

        _updateUI();            
    },

    
    /**
     * Handler for the onLoad of a User object.  This occurs when the User object is initially read
     * from the Finesse server.  Any once only initialization should be done within this function.
     */
    
     _handleUserLoad = function (userevent) {
        _clientLogger.log("_handleUserLoad(): --------");

         _myExtension = _user.getExtension();
        $("#extension").text(_myExtension);
 
		// Add a space after the extension - to prevent it matching a shorter extension (unlikley) - but devices extension includes space and device ID. This is used when searching teh Device List.
        // e.g. "12022 (SEP38ED1855072B)", so we are searching for "12022 " so prevent matching an extension "120223"
        _myExtension = _myExtension + " ";

        _updateUI();
    },
       
    
      
    /**
     *  Handler for all User updates
     */
    _handleUserChange = function(userevent) {
        _clientLogger.log("_handleUserChange(): --------");
        
        if(userevent._data.state === "LOGOUT"){
        _clientLogger.log("_handleUserChange(): User has logged out!");
        _logout();
        }
        
        //_updateUI(); No need to update any more - not changing buttons enablement based on User Change
    };

    _login = function() {
        _clientLogger.log ("_login(): --------");

        var url = _baseURL + _loginPath;

        makeRequest(url, {
            method: 'POST',
            login: 'true',
            contentType: 'application/json',
        }, {
            success: _loginSuccess,
            error: _loginError,
        });
    };
    
    _logout = function () {
        _clientLogger.log ("_logout(): --------");

            var url = _baseURL + _logoutPath;

            makeRequest(url, {
                method: 'POST',
                authToken: 'true',
                contentType: 'application/json',
            }, {
                success: _logoutSuccess,
                error: _logoutError,
            });
    };    

    _deviceList = function () {
        _clientLogger.log ("_deviceList(): --------");
        var url = _baseURL + _deviceListPath;

        makeRequest(url, {
            method: 'GET',
            authToken: 'true',
            contentType: 'application/json',
        }, {
            success: _deviceListSuccess,
            error: _deviceListError,
        });
    };


    _keepalive = function () {
			_clientLogger.log ("_keepalive(): --------");
			_clientLogger.log ("_keepalive(): Checking if we should send Keepalive");
		if (_keepaliveCounter < _keepaliveMaxCounter) {
			_clientLogger.log ("_keepalive(): Keepalive Sent.");

				var url = _baseURL + _keepalivePath;

				makeRequest(url, {
					method: 'PUT',
					authToken: 'true',
					contentType: 'application/json',
				}, {
					success: _keepaliveSuccess,
					error: _keepaliveError,
				});
		}
    };
    
    _startSuppression = function () {
        _clientLogger.log ("_startSuppression(): --------");
        _clientLogger.log ("_startSuppression(): _myDeviceId: _myDeviceId[" + _myDeviceIdCounter + "] = " + (_myDeviceId[_myDeviceIdCounter]));
        _clientLogger.log ("_startSuppression(): _myDeviceId Array: " + _myDeviceId.toString());
        
        var url = _baseURL + _suppressStartPath;
        var contentBody = '{\"deviceID\": ' + _myDeviceId[_myDeviceIdCounter] + '}';
        _clientLogger.log ("_startSuppression(): contentBody: " + _myDeviceId[_myDeviceIdCounter]);

        makeRequest(url, {
            method: 'POST',
            authToken: 'true',
            contentType: 'application/json',
            content: contentBody,
        }, {
            success: _startSuppressionSuccess,
            error: _startSuppressionError
        });
    };    


    _stopSuppression = function () {
        _clientLogger.log ("_stopSuppression(): --------");
        _clientLogger.log ("_stopSuppression(): _myDeviceId: _myDeviceId[" + _myDeviceIdCounter + "] = " + (_myDeviceId[_myDeviceIdCounter]));
        _clientLogger.log ("_stopSuppression(): _myDeviceId Array: " + _myDeviceId.toString());

        var url = _baseURL + _suppressStopPath;
        var contentBody = '{\"deviceID\": ' + _myDeviceId[_myDeviceIdCounter] + '}';
        _clientLogger.log ("_startSuppression(): contentBody: " + _myDeviceId[_myDeviceIdCounter]);

        makeRequest(url, {
            method: 'POST',
            authToken: 'true',
            contentType: 'application/json',
            content: contentBody,
        }, {
            success: _stopSuppressionSuccess,
            error: _stopSuppressionError
        });
    };


    /** @scope finesse.modules.CallRecordingGadget */
    return {
        /**
         * Login to Recorder
         * This is used when a manual button is on the Gadget to assist debugging
         */
        Login : function () {
            $('#loginErrorMsg').hide();
            gadgets.window.adjustHeight();
            _login();
        },

        /**
         * Device List
         * This is used when a manual button is on the Gadget to assist debugging
         */
        DeviceList : function () {
            _clientLogger.log ("deviceList activiated from Button Click --------");
            _deviceList();
        },


        startSuppression : function () {
            _clientLogger.log ("startSuppression activiated from Button Click --------");
            _myDeviceIdCounter = 0;
            _startSuppression();
        },

        stopSuppression : function () {
            _clientLogger.log ("stopSuppression activiated from Button Click --------");
            _myDeviceIdCounter = 0;
            _stopSuppression();
        },

        /**
         * Performs all initialization for this gadget
         */
        init : function () {
            var _clientLogger = finesse.cslogger.ClientLogger;   // declare _clientLogger

            // Initiate the ClientLogs. The gadget id will be logged as a part of the message
            _clientLogger.init(gadgets.Hub, "RedboxCallSuppressionGadget", finesse.gadget.Config);
            _clientLogger.log ("init(): Initializing...");
                
            var cfg = finesse.gadget.Config;
            _util = finesse.utilities.Utilities;

            // Initiate the ClientServices and load the user object. ClientServices are
            // initialized with a reference to the current configuration.
            finesse.clientservices.ClientServices.init(cfg, false);
			
            gadgets.window.adjustHeight();
            
    
            _user = new finesse.restservices.User({
                id: finesse.gadget.Config.id, 
                onLoad : _handleUserLoad,
                onChange : _handleUserChange
            });
            $("#host").text(_host1);
            $("#version").text(_version);

            // Initiate the ContainerServices and add a handler for when the tab is visible
            // to adjust the height of this gadget in case the tab was not visible
            // when the html was rendered (adjustHeight only works when tab is visible)
            // *** Without this the gadget is not correct size on load without refresh the page! ***
            containerServices = finesse.containerservices.ContainerServices.init();
            containerServices.addHandler(finesse.containerservices.ContainerServices.Topics.ACTIVE_TAB, function() {
                _clientLogger.log("Gadget is now visible");  // log to Finesse logger
                // Automatically adjust the height of the gadget to show the html
                gadgets.window.adjustHeight();
            });
            containerServices.makeActiveTabReq();

            _login();
            _clientLogger.log ("init(): Initialization finished.");
        }
    };
}(jQuery));